# frozen_string_literal: true

module ViewComponent
  module VERSION
    MAJOR = 4
    MINOR = 15
    PATCH = 0
    PRE = nil

    STRING = [MAJOR, MINOR, PATCH, PRE].compact.join(".").freeze
  end
end
